use std::io::{stdin,stdout,Write};
use std::fs;
use std::process::{Command};
use serde::{Serialize, Deserialize};

// Declare the modules from these files
mod parse_map; // parse_map.rs
mod output_map; // output_map.rs
mod save; // save.rs

const HELP_STRING: &str = "'help' = output's this help text
'travel' = travel to a given location
";

#[derive(Serialize, Deserialize, Debug)]
struct Coordinates {
	x: i16,
	z: i16,
	map: String,
}
#[derive(Serialize, Deserialize, Debug)]
struct Skills {
	woodcutting: u8,
	mining: u8,
}

#[derive(Serialize, Deserialize, Debug)]
struct Player {
	name: String,
	skills: Skills,
	coordinates: Coordinates,
	inventory: Vec<(String, u64)>,
}

fn clear_screen()
{
	Command::new("clear")
		.status()
		.expect("Failed to clear screen");
}
fn input() -> String{
    let mut s=String::new();
    let _=stdout().flush();
    stdin().read_line(&mut s).expect("Did not enter a correct string");
    if let Some('\n')=s.chars().next_back() {
        s.pop();
    }
    if let Some('\r')=s.chars().next_back() {
        s.pop();
    }
    return s;
}

fn main() 
{
	let debug_mode: bool = true;
	let save_file_path: &str = "data/save.json";
	let (screen_width, screen_height) = termion::terminal_size().unwrap();
	let distance_you_can_see: [i16; 2] = 
	[
		(screen_width as f32 / 2.0) as i16, 
		(screen_height as f32 / 2.0) as i16
	];
	if debug_mode 
	{
		println!("Screen Width: {}, Screen Height: {}", screen_width, screen_height);
	}

	if let Ok(save_file_contents) = fs::read_to_string(save_file_path) // Load save file
	{
		if let Ok(player) = serde_json::from_str::<Player>(save_file_contents.as_str())
		{
			println!("{:?}", player);

			if let Ok((ground_map, above_map)) = parse_map::parse_map(&player.coordinates) // Call the parse map function from parse_map.rs 
			{ // Parse the map file into a vector
				output_map::output_map // Call output_map fuctino from output_map.rs
				(
					&ground_map, 
					&above_map, 
					&player.coordinates, 
					&distance_you_can_see
				); // Output the map
				'game_loop: loop 
				{
					if ! debug_mode 
					{
						clear_screen()
					}
					print!("> ");
					let user_input: String = input();
					if user_input == "help"
					{
						print!("\n{}\n", HELP_STRING);
					}
					else if user_input == "exit"
					{
						if let Err(e) = save::save(&save_file_path, &player)
						{
							eprintln!("Error saving game: {}",e);
							continue;
						}
						print!("Exiting...\n");
						break 'game_loop;
					}
				}
			}
			else 
			{
				eprintln!("Error parsing map files.");
			}
		}
		else
		{
			eprintln!("Failed to parse save file. Invalid JSON:\n{}", save_file_contents);
		}
	}
	else
	{
		println!("No save file, creating new character");
	}
}
